#ifdef _WIN32
#include <windows.h>
#endif 

#include <GL/glew.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>


#include <gl/gl.h>
#include <gl/glu.h>
#include <SDL/SDL.h>


// Globals
#define SCREEN_W	640
#define SCREEN_H	480
#define SCREEN_BPP	32

// Data Types
typedef struct											
{
	bool Visible;										
	bool MouseFocus;									
	bool KeyboardFocus;								
}S_AppStatus;											

S_AppStatus g_AppStatus;									
SDL_Surface *g_Screen;


void ToggleFullscreen(void);
void TerminateApplication(void);							

void CreateWindowGL(int argc, char *argv[]);
void MainLoop();

bool InitGL(SDL_Surface *);								
bool Initialize(void);								
void Deinitialize(void);						
void ReshapeGL(int, int);							
void Draw(SDL_Surface *);									

void Keyboard(SDL_Event &E);
void Mouse ( int button, int state, int x, int y );
void Motion ( int x, int y );
void PassiveMotion ( int x, int y );
void Update(Uint32 Milliseconds);


void CreateWindowGL(int argc, char *argv[])
{
	Uint32		Vflags;														
	g_Screen = NULL;														

	Vflags = SDL_HWSURFACE | SDL_OPENGL | SDL_RESIZABLE;					

// Decide To Always Launch Out Application Fullscreen
#ifdef FULLSCREEN															
	Vflags|= SDL_FULLSCREEN;												
#endif

	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );								
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );							
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );							
	SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 0 );						
	SDL_GL_SetAttribute( SDL_GL_STEREO, 1 );							


	if(!(g_Screen = SDL_SetVideoMode(SCREEN_W, SCREEN_H, SCREEN_BPP, Vflags)))
	{
		printf("Unable to open screen surface: %s\n", SDL_GetError() );		
		exit(1);															
	}
	SDL_FillRect(g_Screen, NULL, SDL_MapRGBA(g_Screen->format,0,0,0,0));	
																		
	ReshapeGL(SCREEN_W, SCREEN_H);										
	SDL_WM_SetCaption("Lession 0", NULL);								
	return;
}

void MainLoop()
{
	bool	bProgramLooping;
	SDL_Event	E;														

	bProgramLooping = true;										
	while(bProgramLooping)								
	{
		if(SDL_PollEvent(&E))											
		{
			switch(E.type)											
			{
			case SDL_QUIT:										
				bProgramLooping = false;									
				break;													
			case SDL_VIDEORESIZE:										
				ReshapeGL(E.resize.w, E.resize.h);					
				break;												
			case SDL_KEYDOWN:								
				Keyboard(E);								
				break;
			case SDL_MOUSEMOTION:
				if(E.motion.state = SDL_PRESSED)
				{
					Motion(E.motion.xrel, E.motion.yrel);
				}else if(E.motion.state = SDL_RELEASED)
				{
					PassiveMotion((int)(E.motion.xrel), (int)(E.motion.yrel));
				}
				break;
			case SDL_MOUSEBUTTONDOWN:
				Mouse ((int)(E.button.button),(int)(E.button.state), E.button.x, E.button.y);
				break;
			/**< Joystick axis motion */
			case SDL_JOYAXISMOTION:											

				break;
			/**< Joystick trackball motion */
			case SDL_JOYBALLMOTION:											

				break;
			/**< Joystick hat position change */
			case SDL_JOYHATMOTION:											

				break;
			/**< Joystick button pressed */
			case SDL_JOYBUTTONDOWN:											

				break;
			case SDL_ACTIVEEVENT:										
				if(E.active.state & SDL_APPACTIVE)					
				{
					if(E.active.gain)								
					{
						g_AppStatus.Visible = true;					
					}
					else											
					{
						g_AppStatus.Visible = false;			
					}
				}
				if(E.active.state & SDL_APPMOUSEFOCUS)			
				{
					if(E.active.gain)							
					{
						g_AppStatus.MouseFocus = true;		
					}
					else										
					{
						g_AppStatus.MouseFocus = false;	
					}
				}
				if(E.active.state & SDL_APPINPUTFOCUS)	
				{
					if(E.active.gain)							
					{
						g_AppStatus.KeyboardFocus = true;
					}
					else								
					{
						g_AppStatus.KeyboardFocus = false;
					}
				}
				break;				
			}
		}
		else								
		{
			if(!g_AppStatus.Visible)
			{
				SDL_WaitEvent(NULL);							
			}

			Draw(g_Screen);							
			SDL_GL_SwapBuffers();							
		}
	}
}

void InitSDL()
{

	if(SDL_Init(SDL_INIT_VIDEO)<0)										
	{
		printf("Unable to open SDL: %s\n", SDL_GetError() );		
		exit(1);													
	}

	if (SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL) < 0)
	{
		printf("Unable to Enable Key Repeat: %s\n", SDL_GetError() );			
	}

	atexit(SDL_Quit);												
}


void TerminateApplication(void)									
{
	static SDL_Event Q;										

	Q.type = SDL_QUIT;										

	if(SDL_PushEvent(&Q) == -1)								
	{
		printf("SDL_QUIT event can't be pushed: %s\n", SDL_GetError() );
		exit(1);											
	}
	return;														
}
// Toggle Fullscreen/Windowed (Works On Linux/BeOS Only)
void ToggleFullscreen(void)													
{
	SDL_Surface *S;												

	S = SDL_GetVideoSurface();									

	if(!S || (SDL_WM_ToggleFullScreen(S)!=1))				
	{
		printf("Unable to toggle fullscreen: %s\n", SDL_GetError() );
	}

	return;	
}
